# LexerTracer.pm (HCT::Lang::LexerTracer)
#
# Copyright 2008, 2009
# Slade Maurer
#
# This file is part of HCT, the Hardware Complexity Tool
#
# HCT is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# HCT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with HCT.  If not, see <http://www.gnu.org/licenses/>.

=head1 NAME

HCT::Lang::LexerTracer  - HCT trace class for C<HCT::Lang::LexerTracer>.

=head1 DESCRIPTION

=cut

package HCT::Lang::LexerTracer;

use strict;
use warnings;

=head1 CONSTRUCTOR

=over 4

=item B<new ()>

Creates a new C<HCT::Lang::Trace> object.

=back

=cut

sub new
{
	my $inv = shift;
	my $class = ref($inv) || $inv;
	my $self = {
		line => undef,
		linepos => undef,
		linenum => undef,
		
		@_,
		
		errors => [],
	};
	bless $self, $class;
	return $self;
}

=head1 METHODS
=cut

=over 4

=cut

=item B<curr_linelen ()> 

Returns current line length.

=cut

sub curr_linelen
{
	my $trace = shift;
	return length (${$trace->{line}}) || 0;
}

=item B<curr_linenum ()> 

Returns current line number.

=cut

sub curr_linenum
{
	my $trace = shift;
	return ${$trace->{linenum}} || 0;
}

=item B<curr_linepos ()> 

Returns current line position.

=cut

sub curr_linepos
{
	my $trace = shift;
	return ${$trace->{linepos}};
}

=item B<curr_line ()> 

Returns current line.

=cut

sub curr_line
{
	my $trace = shift;
	return ${$trace->{line}} || '';
}

=item B<error ($message)>

Creates a new error message and saves it.

=cut

sub error
{
	my $trace = shift;
	my $message = shift;
	my $id = sprintf ("line %d, pos %d: ",$trace->curr_linenum, $trace->curr_linepos);
	
	push @{ $trace->{errors} }, $id . $message;
}

=item B<debug ()> 

Debug.

=cut

sub debug
{
	my $trace = shift;

	my $maxlength = 72;
	my $offset = 0;
	my $line = "";
	my $filler = '~';

	if ($trace->curr_linelen > $maxlength ) {
		$offset = ($trace->curr_linepos > ($maxlength / 2)) ? ($trace->curr_linepos - ($maxlength / 2)) : 0;
		$line = substr($trace->curr_line, $offset, $maxlength);
	}

	else {
		$line = $trace->curr_line;
	}

	my $spaceoffset;

	$line =~ s/^(\s*)//;
	$spaceoffset = length ($1);
	$offset -= $spaceoffset;
	$line =~ s/\s*$//;

	my $lineid = sprintf ("line %d, pos %d: ", $trace->curr_linenum, $trace->curr_linepos);

	print STDERR "${lineid}${line}\n";
	print STDERR $filler x length ($lineid);

	if ($offset > 0) { print STDERR $filler x ($maxlength / 2)	}
	else { print STDERR $filler x ($trace->curr_linepos - $spaceoffset) }

	print STDERR "\^\n";
}

=back

=cut

1;

